#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Grafik_Tkinter.py
# © Claus Albowski
# Quelle des Grundprogramms:
# https://docs.python.org/2.7/library/tkinter.html
# wesentliche Quelle:
# http://infohost.nmt.edu/tcc/help/pubs/tkinter/web
#### Hinweis: Fuer tkFileDialog / tkinter.filedialog muss das
#### jeweilige Modul mit diesem Namen importiert werden!

#### Modellwechsel bei laufendem Programm sollten vermieden werden.
#### Sie werden bei dieser Version verhindert.

#### Importe:
import platform
versionsNummer=platform.python_version()[0]
if versionsNummer=='3': from tkinter import *
else: from Tkinter import *
from simulation import *
#### vordefinierte Dialoge
if versionsNummer=='3':
    import tkinter.filedialog as fileDialog
else:
    import tkFileDialog as fileDialog
from dialoge import * # eigene
#### Spezialfunktionen fuer Terme in funktionen.py
#from funktionen import *
from term import *
from modellfenster import *

class GrafikFenster(Toplevel):
    
    def __init__(self, parent=None, breite=640, hoehe=640):
        '''breite und hoehe definieren die Abmessungen der Zeichenflaeche
        Vorgabe ist 640 x 640 , also je 20 Platz'''
        Toplevel.__init__(self)
        self.parent=parent
        self.title('Grafikfenster')
        self.__breite = breite
        self.__hoehe = hoehe
        self.enthaelt_Ursprung = True
        self.__wertebereich=[]
        self.simu=Simulation.GibSimulation()
  
        top = self.winfo_toplevel()
        ## Menübereich
        self.menuBar = Menu(top)
        top['menu'] = self.menuBar
        #### Simulations-Menu
        self.simuSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Simulation', font='Sans 10 bold', menu=self.simuSubMenu)
        self.simuSubMenu.add_command(label=u'Wähle Faktor(en)', font='Sans 10 bold', command=self.onWaehleFaktoren)
        self.simuSubMenu.add_command(label='Start', font='Sans 10 bold', command=self.zeichne_Kurve, state=DISABLED)
        self.simuSubMenu.add_command(label='Simulationsmethode', font='Sans 10 bold', command=self.onWaehleMethode)
        #### Darstellungs-Menue
        self.darstellungSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Darstellung', font='Sans 10 bold', menu=self.darstellungSubMenu)
        self.darstellungSubMenu.add_command(label=u'Fenstergröße ändern', font='Sans 10 bold', command=self.onFenster)
        self.darstellungSubMenu.add_command(label='Wertebereich y-Achse setzen', font='Sans 10 bold', command=self.onWertebereich)
        self.darstellungSubMenu.add_command(label='Eine Größe skalieren', font='Sans 10 bold', command=self.onSkalieren)
        self.darstellungSubMenu.add_command(label='Verlaufsdiagramm speichern', font='Sans 10 bold', command=self.onBild)
        #### Tabellen-Menue
        self.tabellenSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Tabelle', font='Sans 10 bold', menu=self.tabellenSubMenu)
        self.tabellenSubMenu.add_command(label='Tabelle anzeigen', font='Sans 10 bold', command=self.onTabelleZeigen)
        self.tabellenSubMenu.add_command(label='Tabelle in csv-Datei', font='Sans 10 bold', command=self.onTabelleSpeichern)
        #### Hilfe-Menu
        self.hilfeSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Hilfe', font='Sans 10 bold', menu=self.hilfeSubMenu)
        self.hilfeSubMenu.add_command(label='Hilfe', font='Sans 10 bold', command=self.onZeigeHilfeText)

        ## Anlegen der Attribute        
        self.__simulation=Simulation.GibSimulation(self)        
        ## Tk-Variable [Daten für Widgets]
        self.nEdit = IntVar()         ## Anzahl Schritte
        self.nEdit.set(10)
        self.endEdit = DoubleVar()         ## Endzeitpunkt
        self.endEdit.set(10.0)
        self.dtEdit = DoubleVar()        ## Schrittweite
        self.dtEdit.set(1.0)
        self.istPhasendiagramm=IntVar() # implizit 0, besser aber:
        self.istPhasendiagramm.set(0)
        self.xFaktor = 't'     # Faktor, der in x-Richtung dargestellt wird
        self.yFaktoren = []  #  Faktor(en), die in y-Richtung dargestellt werden
        self.spezielleSkalierung = None # ermoeglicht die Skalierung einer Darstellung
        ## sonst (Nummer der Kurve, Skalierungsfaktor)
        self.farbNamen=['schwarz','rot','blau','magenta','gruen','weiss']
        self.farbeZuNamen={'schwarz':'#000','rot':'#f00','blau':'#00f','magenta':'#f0f','gruen':'#0b0','weiss':'#fff'}
        self.verfahren = StringVar()
        self.verfahren.set('Runge-Kutta')
        ## Definition von Mausereignissen fuer das Fenster
        self.winfo_toplevel().bind("<Control-f>", self.onWaehleFaktoren)
        self.winfo_toplevel().bind("<Control-m>", self.onWaehleMethode)
        self.winfo_toplevel().bind("<Control-p>", self.onBild)
        self.winfo_toplevel().bind("<Control-t>", self.onTabelleZeigen)
        self.winfo_toplevel().bind("<Control-z>", self.zeichne_Kurve)
        ## Oberflaeche erzeugen
        self.createWidgets()

    def createWidgets(self):
        '''Einsatz des grid - Geometriemanagers'''
       
        self.n_Label = Label(self, text=" Anzahl Schritte ", font='Sans 10')
        self.n_Label.grid(row=0,column=0)
        
        self.nEingabe = Entry(self,textvariable=self.nEdit, font='Sans 10 bold')
        self.nEingabe.grid(row=0,column=1)
        # Ereignisse:
        self.nEingabe.bind("<FocusOut>", self.OnAnzahlGeaendert)
        self.nEingabe.bind("<Return>", self.OnAnzahlGeaendert)

        self.end_Label = Label(self, text=" Endwert ", font='Sans 10')
        self.end_Label.grid(row=1,column=0)
        
        self.endEingabe = Entry(self,textvariable=self.endEdit, font='Sans 10 bold')
        self.endEingabe.grid(row=1,column=1)
        # Ereignisse:
        self.endEingabe.bind("<FocusOut>", self.OnEndeGeaendert)
        self.endEingabe.bind("<Return>", self.OnEndeGeaendert)

        self.dt_Label = Label(self, text=" Schrittweite ", font='Sans 10')
        self.dt_Label.grid(row=2,column=0)
        
        self.dtEingabe = Entry(self,textvariable=self.dtEdit, font='Sans 10 bold')
        self.dtEingabe.grid(row=2,column=1)
        # Ereignisse:
        self.dtEingabe.bind("<FocusOut>", self.OnSchrittweiteGeaendert)
        self.dtEingabe.bind("<Return>", self.OnSchrittweiteGeaendert)

        self.phasen_Check = Checkbutton(self,
                                           text='Phasendiagramm',
                                           variable=self.istPhasendiagramm)
        self.phasen_Check.grid(row=3,column=0)

        self.verfahrenLabel = Label(self, textvariable=self.verfahren, font='Sans 10 bold')
        self.verfahrenLabel.grid(row=3,column=1)

        self.berechnen_Button = Button(self,
                                       text="Berechnen",
                                       command=self.zeichne_Kurve,
                                       state=DISABLED,
                                       font='Sans 10 bold')
        self.berechnen_Button.grid(row=0,column=2,rowspan=3)

        #### Zeichenfenster
        self.zeichenfenster = Canvas(self,width=self.__breite,height=self.__hoehe,bg='white')
        self.zeichenfenster.grid(row=4,column=0, columnspan=3)

    #### Ereignisbehandlung fuer die Edits
    def OnAnzahlGeaendert(self, evt=None):
        if self.nEdit.get()>1:
            self.endEdit.set(self.nEdit.get()*self.dtEdit.get())
        else: # Wert sinnlos => Anzahl wieder herstellen
            self.nEdit.set(int(self.endEdit.get()/self.dtEdit.get()))
            
    def OnEndeGeaendert(self, evt=None):
        anzahl=int(self.endEdit.get()/self.dtEdit.get())
        if anzahl>1:
            self.nEdit.set(anzahl)
        else: # Wert sinnlos => Ende wieder herstellen
            self.endEdit.set(self.nEdit.get()*self.dtEdit.get())

    def OnSchrittweiteGeaendert(self, evt=None):
        anzahl=int(self.endEdit.get()/self.dtEdit.get())
        if anzahl>1:
            self.nEdit.set(anzahl)
        else: # Wert sinnlos => Schrittweite wieder herstellen
            self.dtEdit.set(self.endEdit.get()/self.nEdit.get())
            

    #### Simulations-Menu
    def onWaehleFaktoren(self, evt=None):
        '''Die Methode ermoeglicht die Auswahl der darzustellenden Faktoren.
        Wird nur einer ausgewaehlt, wird fuer x die Zeit (t) gewaehlt.'''
        if self.istPhasendiagramm.get():
            # Phasendiagramm
            self.Meldung(u'Faktoren einzeln wählen!')
            gewaehlteFaktoren=self.FaktorenAuswahlDialog()
            if gewaehlteFaktoren==None: return # cancel angeklickt
            self.xFaktor=gewaehlteFaktoren[0]
            gewaehlteFaktoren=self.FaktorenAuswahlDialog()
            if gewaehlteFaktoren==None: return # cancel angeklickt
            self.yFaktoren=gewaehlteFaktoren[0:1]
        else:
            # Zeitdiagramm
            gewaehlteFaktoren=self.FaktorenAuswahlDialog()
            if gewaehlteFaktoren==None: return # cancel angeklickt
            self.xFaktor='t'
            self.yFaktoren=gewaehlteFaktoren
            if len(self.yFaktoren)>5:
                self.Meldung('maximal 5 Kurven!')
                self.yFaktoren=self.yFaktoren[:5]
        ## Menue-Punkt und Button aktivieren
        self.berechnen_Button.config(state=NORMAL)
        self.simuSubMenu.entryconfig(2,state=NORMAL)
        self.spezielleSkalierung = None # ggf reset notwendig

    def onWaehleMethode(self, evt=None):
        '''Es kann zwischen den Simulationsverfahren
        einfaches Euler-Verfahren, erweitertes Eulerverfahren und Runge-Kutta (fehlt noch)
        ausgewaehlt werden.'''
        dialog=AuswahlDialog(self,u'Faktor(en) wählen',
                             ['einfach Euler', 'erweitert Euler','Runge-Kutta'],
                             einzeln=True)
        if dialog.result==None: return # bei cancel None
        verfahren=dialog.result
        self.__simulation.SetzeSimulationsMethode(verfahren)
        self.verfahren.set(verfahren)

    #### Darstellungs-Menue
    def onFenster(self):
        dialog=N_EingabenDialog(self,
                                u'Fenstergröße ändern',
                                [u'neue Breite (>=240)\nStandard 640',
                                 u'neue Fensterhöhe (>=240)\nStandard 640'],
                                [str(self.__breite),str(self.__hoehe)])
        if dialog.result==None: return # cancel ausgewaehlt
        try:
            breite=int(string_in_Zahl(dialog.result[0]))
            hoehe=int(string_in_Zahl(dialog.result[1]))
        except ValueError as e:
            self.Meldung('Fehler: '+str(e), 'Achtung')
            return # falsche Werte
        self.__breite = max(240,breite)
        self.__hoehe = max(240,hoehe)
        self.zeichenfenster.grid_forget()
        self.zeichenfenster = Canvas(self,width=self.__breite,height=self.__hoehe,bg='white')
        self.zeichenfenster.grid(row=4,column=0, columnspan=3)      

    def onWertebereich(self, evt=None):
        dialog=N_EingabenDialog(self,'Wertebereich y-Achse setzen',
                                ['von','bis'],
                                [0,0])
        if dialog.result==None: return # cancel ausgewaehlt
        if dialog.result==['0','0']:
            self.__wertebereich=[]
            return
        try:
            self.__wertebereich=float(dialog.result[0]),float(dialog.result[1])
        except ValueError as e:
            self.Meldung('Fehler: '+str(e), 'Achtung')

    def onSkalieren(self, evt=None):
        dialog1=AuswahlDialog(self, u'Namen der Größe eingeben', self.yFaktoren,True)
        if dialog1.result==None: return # Abbruch
        index=self.yFaktoren.index(dialog1.result)+1 # hier 1 basierend !!!
        dialog2=EingabeDialog(self, 'Faktor eingeben', 'Skalieren mit:')
        if dialog2.result==None: return # Abbruch
        try: faktor=float(dialog2.result)
        except ValueError as e:
            self.Meldung('Zahl eingeben!')
            return
        self.spezielleSkalierung = (index,faktor)

    def onBild(self, evt=None):
        modName=self.simu.GibModellNamen()
        if modName==None: modName=""
        dateiname=fileDialog.asksaveasfilename(parent=self,
                                                 initialfile="Verlaufsdiagramm_"+modName+".ps",
                                                 defaultextension='ps',
                                                 initialdir='Bilder',
                                                 title="Verlaufsdiagramm speichern")
        if dateiname in ['',()]:
            self.Meldung('Bild nicht gespeichert','Achtung !')
        else:
            self.zeichenfenster.postscript(file=dateiname)

    #### Tabellen-Menue
    def onTabelleZeigen(self, evt=None):
        '''Es werden hoechstens 31 Werte angezeigt (30 Schritte)'''
        def i_te_Zeile(i,werte):
            zeile=[]
            for faktorwerte in werte:
                zeile.append(faktorwerte[i])
            return zeile
        faktorNamen=[self.xFaktor]+self.yFaktoren
        werte=self.HoleTabellendaten(faktorNamen)
         # Laenge begrenzen:
        schrittweite = int(len(werte[0]) / 32 + 1)
        wertetabelle=[]
        for i in range(len(werte[0])):
            if (i%schrittweite)==0:
                wertetabelle.append(i_te_Zeile(i,werte))
        dialog=TabellenAusgabeDialog(self,
                                     u'Tabellenausgabe',
                                     [faktorNamen]+wertetabelle)

    def HoleTabellendaten(self, faktorNamen):
        '''die zurueckgegebene Liste ist eine Liste
        der Listen aller Werte zu einem Faktor'''
        faktoren=[]
        for faktorName in faktorNamen:
            faktoren.append(Faktor.FaktorZuNamen(faktorName))
        werte=[]
        for faktor in faktoren:
            werte.append(faktor.GibAlleWerte())
        return werte

    def onTabelleSpeichern(self):
        '''Alle Werte werden in eine csv-Datei gespeichert, die mit einer
        Tabellenkalkulation geoffnet werden kann.
        Beim Import beachten:
        Wegen des Zahlenformats Sprache Englisch einstellen (3.5 -> 3,5)'''
        if self.yFaktoren==[]:
            self.Meldung('Keine Daten zum Schreiben vorhanden!', 'Achtung !')
            return
        faktorNamen=[self.xFaktor]+self.yFaktoren
        werte=self.HoleTabellendaten(faktorNamen)
        modName=self.simu.GibModellNamen()
        if modName==None: modName=""
        dateiname='Tabellendaten_'+modName+'.csv'
        dateiname=fileDialog.asksaveasfilename(parent=self,
                                                 initialfile=dateiname,
                                                 defaultextension='csv',
                                                 initialdir='Tabellendaten',
                                                 title="Tabellendaten speichern")
        if not(dateiname in ['', ()]):
            spalten=len(faktorNamen)
            f=open(dateiname,'w')
            ausgabe=''
            for i in range(spalten):
                ausgabe += faktorNamen[i]
                if i < spalten-1:
                    ausgabe += ';'
                else:
                    ausgabe += '\n'
            f.write(ausgabe)            
            for i in range(len(werte[0])):
                ausgabe=''
                for j in range(spalten):
                    ausgabe += str(werte[j][i])
                    if j< spalten-1:
                        ausgabe += ';'
                if i < len(werte[0])-1:
                    ausgabe += '\n'
                f.write(ausgabe)
            f.close()
        else:
            self.Meldung('Keine Daten geschrieben!', 'Info')

    #### ----------------------------------------------
    def FaktorenAuswahlDialog(self, faktorTyp=None):
        faktorNamen=Simulation.GibSimulation().GibFaktorNamen(faktorTyp)
        dialog=AuswahlDialog(self,u'Faktor(en) wählen',faktorNamen)
        return dialog.result # bei cancel None

    def TesteYFaktoren(self):
        if self.yFaktoren == []: return False
        alleNamen=Simulation.GibSimulation().GibAlleFaktorNamen()
        for name in self.yFaktoren:
            if not (name in alleNamen): return False
        return True
        
    def zeichne_Kurve(self, evt=None):
        if not self.TesteYFaktoren():
            self.Meldung(u'Faktoren neu Wählen!')
            return
        ## Hilfsvariable zur leichteren Anwendung
        simu=self.__simulation
        zf=self.zeichenfenster
        zf.delete('Zeichnung') # alte Darstellung loeschen
        werte=[] # werden [(x0, y1(x0), ggf weitere), ...]
        self.ursprung = (20, self.__hoehe/2) # Dummy-Wert
        simu.Setze_dt(self.dtEdit.get())
        simu.SetzeAnzahlSchritte(self.nEdit.get())
        werte = simu.Starte_Simulation([self.xFaktor]+self.yFaktoren)
        if werte==[]:return
        x_koordinaten = self.Hole_x_Koordinaten(werte)
        x_von_bis = self.MinMax(x_koordinaten)
        if self.__wertebereich!=[]:
            y_von_bis=self.__wertebereich[0],self.__wertebereich[1]
        else:
            y_von_bis = self.MinMax(self.Hole_y_Koordinaten(werte))
            if len(self.yFaktoren)>1:
                for i in range(1,len(self.yFaktoren)):
                    mi,ma=self.MinMax(self.Hole_y_Koordinaten(werte,i+1))
                    y_von_bis = min(mi,y_von_bis[0]) , max(ma,y_von_bis[1]) 
        #### Daten fuer die Darstellung berechnen:
        zb_x = x_von_bis[1]-x_von_bis[0]
        zb_y = y_von_bis[1]-y_von_bis[0]
        if zb_y==0:  # verhindert /0 bei konstanten Werten
            zb_y=2
            y_von_bis=y_von_bis[0]-1,y_von_bis[1]+1
        self.enthaelt_Ursprung =  y_von_bis[0]*y_von_bis[1]<=0
        sf_x = lambda x: (self.__breite-40)*x/zb_x
        sf_y = lambda y: (self.__hoehe-40)*y/zb_y
        self.ursprung = (20+int(sf_x(-x_von_bis[0])), 20+int(sf_y(y_von_bis[1])))
        self.KoordinatensystemZeichnen(zf,x_von_bis[0],x_von_bis[1],y_von_bis[0],y_von_bis[1])
        self.GraphenZeichnen(zf,werte,x_koordinaten,sf_x,sf_y)

    def KoordinatensystemZeichnen(self,zf,x_Wert_unten,x_Wert_oben,y_Wert_unten,y_Wert_oben):
        zf.create_line(0.0,self.ursprung[1],self.__breite,self.ursprung[1],tag='Zeichnung',dash=(5,1),width=2)
        zf.create_line(self.ursprung[0],0.0,self.ursprung[0],self.__hoehe,tag='Zeichnung',dash=(5,1),width=2)
        ## Beschriftung y-Achse:
        if self.spezielleSkalierung!=None: # Skalierung textlich anzeigen
            speziell,faktor=self.spezielleSkalierung
            farbe=self.farbeZuNamen[self.farbNamen[speziell-1]]
            if faktor==1: skalText=''
            else: skalText=self.yFaktoren[speziell-1]+" skaliert mit Faktor "+str(faktor)
            zf.create_text(150, 15, text=skalText,anchor=W,fill=farbe,tag='Zeichnung', font="Sans 11 italic")
        if self.enthaelt_Ursprung: xFuerY = self.ursprung[0]
        else: xFuerY = 10 # Abstand vom linken Rand
        zf.create_line(xFuerY-5, 20, xFuerY+5, 20, tag='Zeichnung')
        zf.create_line(xFuerY-5, self.__hoehe/2, xFuerY+5, self.__hoehe/2, tag='Zeichnung')
        zf.create_line(xFuerY-5, self.__hoehe-20, xFuerY+5, self.__hoehe-20, tag='Zeichnung')
        # Zahldarstellung kuerzen und in String verwandeln:
        y_Wert_mitte=(y_Wert_unten+y_Wert_oben)/2
        y_unten,y_mitte,y_oben=self.kurzStr(y_Wert_unten),self.kurzStr(y_Wert_mitte),self.kurzStr(y_Wert_oben)
        zf.create_text(xFuerY+10, 15, text=y_oben,anchor=W,tag='Zeichnung', font="Sans 11 bold")
        zf.create_text(xFuerY+10, self.__hoehe/2-10,text=y_mitte,anchor=W,tag='Zeichnung', font="Sans 11 bold")
        zf.create_text(xFuerY+10, self.__hoehe-25,text=y_unten,anchor=W,tag='Zeichnung', font="Sans 11 bold")
        ## Beschriftung x -Achse, ggf ohne die Achse:
        zf.create_line(20, self.ursprung[1]-7, 20, self.ursprung[1]+7, width=2, tag='Zeichnung')
        zf.create_line(self.__breite/2-10, self.ursprung[1]-7, self.__breite/2-10, self.ursprung[1]+7, width=2, tag='Zeichnung')
        zf.create_line(self.__breite-20, self.ursprung[1]-7, self.__breite-20, self.ursprung[1]+7, width=2, tag='Zeichnung')
        if self.enthaelt_Ursprung: yText = self.ursprung[1]+5
        else: yText = self.__hoehe-15
        # Zahldarstellung kuerzen und in String verwandeln:
        x_Wert_mitte=(x_Wert_unten+x_Wert_oben)/2
        x_unten,x_mitte,x_oben=self.kurzStr(x_Wert_unten),self.kurzStr(x_Wert_mitte),self.kurzStr(x_Wert_oben)
        zf.create_text(15, yText, text=x_unten,anchor=N,tag='Zeichnung', font="Sans 11 bold")
        zf.create_text(self.__breite/2-12, yText, text=x_mitte,anchor=N,tag='Zeichnung', font="Sans 11 bold")
        zf.create_text(self.__breite-25, yText, text=x_oben,anchor=N,tag='Zeichnung', font="Sans 11 bold")
        
        
    def GraphenZeichnen(self,zf,werte,x_koordinaten,sf_x,sf_y):
        #### Graph(en) zeichnen
        anzahl=len(self.yFaktoren)
        farben=self.farbNamen[:anzahl]
        for i in range(anzahl):
            bildschirmkoordinaten = self.koord_zu_bildschirm(
                x_koordinaten,
                self.Hole_y_Koordinaten(werte,i+1),
                sf_x,sf_y)
            zf.create_line(bildschirmkoordinaten,fill=self.farbeZuNamen[farben[i]],width=2,tag='Zeichnung')
            zf.create_text(60,60+i*20,
                           text=self.yFaktoren[i][:12],
                           fill=self.farbeZuNamen[farben[i]],tag='Zeichnung', font="Sans 12 bold")
        self.__neuesModell = False

    #### Hilfsmethoden für die Darstellung
    
    def koord_zu_bildschirm(self,x_koordinaten,y_koordinaten,sf_x,sf_y):
        '''Die Transformation der Koordinaten in Bildschirmkoordinaten passt
        auch die Koordinaten an die y-Richtung der Widgets an.'''
        assert len(x_koordinaten)==len(y_koordinaten), "Koordinatenfehler!"
        bildschirm_koordinaten = []
        for i in range(len(x_koordinaten)):
            bildschirm_koordinaten.append(sf_x(x_koordinaten[i])+self.ursprung[0])
            bildschirm_koordinaten.append(-sf_y(y_koordinaten[i])+self.ursprung[1])
        return bildschirm_koordinaten

    def kurzStr(self, zahl):
        '''gibt eine kurze String-Darstellung der Zahl zurueck'''
        if (abs(zahl)<1e5) and float(zahl).is_integer(): return str(int(zahl))
        exponentiell=False
        if (abs(zahl)>1e5): exponentiell=True
        if (abs(zahl)<0.0001): exponentiell=True
        if not exponentiell:
            if zahl<1: return round(zahl*10000)/10000.0
            else: return str(round(10*zahl)/10.0)
        t="%.2e" % zahl
        zahlBas,e,zahlexp=t.partition('e')
        return zahlBas+e+zahlexp # String erzwingen

    #### Hilfsfunktionen
    def wendeAnAuf(self, funktion, liste):
        '''Versionsunabhaengigkeit von map herstellen'''
        ergebnis=[]
        for element in liste:
            ergebnis.append(funktion(element))
        return ergebnis

    def hol_i(self, i): return (lambda t: t[i]) # holt i-ten Wert

    def Hole_x_Koordinaten(self, werte):
        '''Die Funktion holt aus den Werten alle x-Koordinaten'''
        return self.wendeAnAuf(self.hol_i(0),werte)

    def Hole_y_Koordinaten(self, werte, index=1):
        '''Die Funktion holt aus den Werten alle y-Koordinaten,
        mehrere y-Koordinaten sind zulaessig:
        index: 1.. fuer 1. , 2. ... y-Koordinate'''
        yWerte_zu_i=self.wendeAnAuf(self.hol_i(index),werte)
        if self.spezielleSkalierung!=None:
            speziell,faktor=self.spezielleSkalierung
            if speziell==index:
                if speziell in range(len(self.yFaktoren)+1): # Nummer der Kurve ok
                    yWerte_zu_i=self.wendeAnAuf(lambda x: faktor*x,yWerte_zu_i)
        return yWerte_zu_i

    def MinMax(self, koordinaten):
        '''Die Funktion liefert das Paar aus dem kleinsten und dem groessten
        Koordinatenwert zurueck.'''
        return min(koordinaten), max(koordinaten)

    #### Hilfe-Menu
    def onZeigeHilfeText(self):
        TextAusgabeDialog(
            self,
            u'Hilfe zum Grafikfenster',
            u'Starten der Simulation',
            [u'Vor dem Starten der Simulation'+
             u' die darzustellenden Größen auswählen.',
             u' (-> Simulation -> Faktor(en) wählen)\n',
             u'wähle mit <Strg> - linker Mausklick\n\n',
             u'Wird nicht das Phasendiagramm gewählt',
             u' (-> Checkbox),',
             u' steht die x-Achse automatisch für t.'])

    def Meldung(self, text, title='Info'):
        MeldungDialog(self, title, text)

    def quit(self):
        '''verhindert das Schliessen des Fensters'''
        self.withdraw()

#### -----------------------------------------------------------------------
if __name__=='__main__':
    root = Tk()
    grafikfenster = GrafikFenster(root)
    grafikfenster.mainloop()
    try:
        root.destroy()
    except Exception: pass
        
